// user[hœ삷L[{[hhCo
// 2008/01/31
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <linux/input.h>
#include <linux/uinput.h>
#include <sys/select.h>
#include <inttypes.h>
#include <asm/types.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include "keydriver.h"
#include "cpu_dependency.h"

// global
static int g_uinput_fd = -1;

#define EVDEV_MINORS	32
static int g_envdev_key_fds[EVDEV_MINORS];
static int g_envdev_key_fds_count = 0;

static bool is_keyboard_device(int fd)
{
	uint8_t evtype_bitmask[EV_MAX/8 + 1];
	struct input_id devinfo;
	
	// check bustype
	if (ioctl(fd, EVIOCGID, &devinfo) != 0)
	{
		//Ȃ
		fprintf(stderr, "error: EVIOCGID ioctl failed.\n");
		return false;
	}

	// allow USB, PS/2, ADB
	switch (devinfo.bustype)
	{
	case BUS_USB:
	case BUS_I8042:
	case BUS_ADB:
		break;					// ok
	default:
		return false;			// unmatch bus type
	}

	// check ev_bit
	if (ioctl(fd, EVIOCGBIT(0, sizeof(evtype_bitmask)), evtype_bitmask) > 0)
	{
		// EV_SYN, EV_KEY, EV_REP Ȃ炨
		if (test_bit(EV_SYN, evtype_bitmask) && test_bit(EV_KEY, evtype_bitmask) && test_bit(EV_REP, evtype_bitmask))
		{				
			return true;
		}
	}

	return false;
}

static int open_key_device(int dev_num)
{
	char path[128];
	int fd, version;
	int i;
	char* endev_path[] = {"/dev/input/event%d", "/dev/event%d"};


	for (i = 0; i < array_size(endev_path); i++)
	{
		sprintf(path, endev_path[i], dev_num);
		fd = open(path, O_RDONLY | O_NDELAY);
		if (fd >= 0)
			break;
	}	
	if (-1 == fd)
	{
		return -1;
	}

	if (ioctl(fd, EVIOCGVERSION, &version) == -1)
	{
		fprintf(stderr, "error: EVIOCGVERSION ioctl failed\n");
		close(fd);
		return -1;
	}

	//evdeṽfoCXŖȂ舵Ȃ
	if (version != EV_VERSION)
	{
		close(fd);
		return -1;
	}
	return fd;
}

// I[vfoCX̌Ԃ
static int open_keyboard()
{
	int i, fd;
	
	for (i = 0;i < EVDEV_MINORS; i++)
	{
		fd = open_key_device(i);
		if (fd != -1)
		{
			if (is_keyboard_device(fd))
			{
				g_envdev_key_fds[g_envdev_key_fds_count++] = fd;
			}
			else
			{
				close(fd);
			}
		}
	}
	return g_envdev_key_fds_count;
}

static void close_keyboard()
{
	int i;
	for (i = 0;i < g_envdev_key_fds_count; i++)
	{
		close(g_envdev_key_fds[g_envdev_key_fds_count]);
		g_envdev_key_fds[i] = 0;
	}
	g_envdev_key_fds_count = 0;
}

// L[R[ho͗p̃L[{[hj
static void destroy_uinput_keyboard()
{
	if (g_uinput_fd != -1)
	{
		ioctl(g_uinput_fd, UI_DEV_DESTROY);
		close(g_uinput_fd);
		g_uinput_fd = -1;
	}
}

// L[R[ho͗p̃L[{[h쐬
static int create_uinput_keyboard()
{
	struct uinput_user_dev uinp = {{0}};
	int i=0;
	int ret;
	const char* uinput_path[] = {"/dev/input/uinput", "/dev/uinput"};

	destroy_uinput_keyboard();
	// input deviceJ
	for (i = 0; i < array_size(uinput_path); i++)
	{
		g_uinput_fd = open(uinput_path[i], O_RDWR);
		if (g_uinput_fd >= 0)
			break;
	}

	if (g_uinput_fd <= 0)
	{
		return -1;
	}
	
	strncpy(uinp.name, "mayu uinpt", UINPUT_MAX_NAME_SIZE);
	uinp.id.vendor = 1;
	uinp.id.bustype = BUS_I8042;
	//	uinp.id.bustype = BUS_USB;
	uinp.id.product = 1;
	uinp.id.version = 4;
	// uinput device쐬
	ret = ioctl(g_uinput_fd, UI_SET_EVBIT, EV_KEY);
	ioctl(g_uinput_fd, UI_SET_EVBIT, EV_SYN);
	ioctl(g_uinput_fd, UI_SET_EVBIT, EV_REP);
	ioctl(g_uinput_fd, UI_SET_EVBIT, EV_REL);
	ioctl(g_uinput_fd, UI_SET_RELBIT, REL_X);
	ioctl(g_uinput_fd, UI_SET_RELBIT, REL_Y);
	for (i=0; i < KEY_MAX; i++) {
		ioctl(g_uinput_fd, UI_SET_KEYBIT, i);
	}
	ioctl(g_uinput_fd, UI_SET_KEYBIT, BTN_MOUSE);
	//	ioctl(g_uinput_fd, UI_SET_KEYBIT, BTN_TOUCH);
	ioctl(g_uinput_fd, UI_SET_KEYBIT, BTN_MOUSE);
	ioctl(g_uinput_fd, UI_SET_KEYBIT, BTN_LEFT);
	ioctl(g_uinput_fd, UI_SET_KEYBIT, BTN_MIDDLE);
	ioctl(g_uinput_fd, UI_SET_KEYBIT, BTN_RIGHT);
	ioctl(g_uinput_fd, UI_SET_KEYBIT, BTN_FORWARD);
	ioctl(g_uinput_fd, UI_SET_KEYBIT, BTN_BACK);
	
	write(g_uinput_fd, &uinp, sizeof(uinp));

	ret = ioctl(g_uinput_fd, UI_DEV_CREATE);
	if (ret)
	{
		fprintf(stderr, "error: failed to open uinput device.\n");
		return -1;
	}
	return 0;
}

// sȂG[R[hAȂ0Ԃ
int open_keyboard_device()
{
	int ret = open_keyboard(); // I[vɐfoCX̌Ԃ
	if (ret <= 0)
		return -1;
	
	ret = create_uinput_keyboard(); // 0Ԃ
	if (ret < 0)
	{
		close_keyboard_device();
		return -1;
	}

	return 0;
}

void close_keyboard_device()
{
	keyboard_grab_onoff(false);
	destroy_uinput_keyboard();
	close_keyboard();
}

//Ȃ0Ԃ
int send_input_event(int type, int code, int value)
{
	int result;
	input_event event = {{0}};

	if (g_envdev_key_fds_count <= 0 && g_uinput_fd <= 0)
		return -1;
	
	gettimeofday(&event.time, NULL);
	event.type = type;
	event.code = code;
	event.value = value;
	
	result = write(g_uinput_fd, &event, sizeof(event));
	return (result < 0 ? errno :
			(result < sizeof(event) ? -1 : 0));
}

bool send_keyboard_event(int code, KEY_EVENT_VAL value)
{
	if (g_envdev_key_fds_count <= 0 && g_uinput_fd <= 0)
		return false;
	
	if (send_input_event(EV_KEY, code, value) < 0)
		return false;

	send_input_event(EV_SYN, SYN_REPORT, 0); // G[Ă
	
	return true;
}

// error code
bool receive_keyboard_event(struct input_event* event)
{
	struct input_event revent = {{0}};
	fd_set set;
	int i, max_fd = 0;
	
	bool loop_stop = false;

	if (g_envdev_key_fds_count <= 0 && g_uinput_fd <= 0)
		return false;
	
	while (!loop_stop)
	{
		// o^ & fdmaxT
		FD_ZERO(&set);				//
		for (i = 0;i < g_envdev_key_fds_count; i++)
		{
			FD_SET(g_envdev_key_fds[i], &set);
			max_fd = max(max_fd, g_envdev_key_fds[i]);
		}

		// f[^܂őҋ@
		select(max_fd + 1, &set, NULL, NULL, NULL);

		// f[^܂ꂽfoCXǂݍ
		for (i = 0;i < g_envdev_key_fds_count; i++)
		{
			if (FD_ISSET(g_envdev_key_fds[i], &set))
			{
				if (read(g_envdev_key_fds[i], &revent, sizeof(revent)) < sizeof(revent))
					return false;

				// L[{[hCxgȂhandler֒ʒm
				if (revent.type == EV_KEY)
				{
					loop_stop = true;
					*event = revent;
					break;
				}
				else if (revent.type == EV_SYN)	// 
				{
				}
 				else if (revent.type == EV_MSC)	// 
 				{
 				}
				else
				{
					// L[{[hCxgȊÓÂ܂܏o
					write(g_uinput_fd, &revent, sizeof(revent));
				}
			}
		}
	}
	
	return true;
}

void keyboard_grab_onoff(bool onoff)
{
	int i;
	
	if (g_envdev_key_fds_count <= 0 && g_uinput_fd <= 0)
		return;
	
	for (i = 0; i < g_envdev_key_fds_count; i++)
	{
		if (ioctl(g_envdev_key_fds[i], EVIOCGRAB, (int)onoff) == -1)	//grabJn
			fprintf(stderr, "error: EVIOCGRAB ioctl failed\n");
	}
}
